extern bool_t xdr_domainname __P((XDR *, char *));
extern bool_t xdr_peername __P((XDR *, char *));
extern bool_t xdr_datum __P((XDR *, datum *));
extern bool_t xdr_mapname __P((XDR *, char *));
extern bool_t xdr_ypreq_key __P((XDR *, struct ypreq_key *));
extern bool_t xdr_ypreq_nokey __P((XDR *, struct ypreq_nokey *));
extern bool_t xdr_yp_inaddr __P((XDR *, struct in_addr *));
extern bool_t xdr_ypbind_binding __P((XDR *, struct ypbind_binding *));
extern bool_t xdr_ypbind_resptype __P((XDR *, enum ypbind_resptype *));
extern bool_t xdr_ypbind_resp __P((XDR *, struct ypbind_resp *));
extern bool_t xdr_ypresp_val __P((XDR *, struct ypresp_val *));
extern bool_t xdr_ypbind_setdom __P((XDR *, struct ypbind_setdom *));
extern bool_t xdr_ypresp_key_val __P((XDR *, struct ypresp_key_val *));
extern bool_t xdr_ypresp_all_seq __P((XDR *, u_long *));
extern bool_t xdr_ypresp_master __P((XDR *, struct ypresp_master *));
extern bool_t xdr_ypmaplist_str __P((XDR *, char *));
extern bool_t xdr_ypmaplist __P((XDR *, struct ypmaplist *));
extern bool_t xdr_ypresp_maplist __P((XDR *, struct ypresp_maplist *));
extern bool_t xdr_ypresp_order __P((XDR *, struct ypresp_order *));
